TIFF_VERSION_CLASSIC = 42
TIFF_VERSION_BIG = 43
TIFF_BIGENDIAN = 19789
TIFF_LITTLEENDIAN = 18761
MDI_LITTLEENDIAN = 20549
MDI_BIGENDIAN = 17744
TIFFTAG_SUBFILETYPE = 254
FILETYPE_REDUCEDIMAGE = 1
FILETYPE_PAGE = 2
FILETYPE_MASK = 4
TIFFTAG_OSUBFILETYPE = 255
OFILETYPE_IMAGE = 1
OFILETYPE_REDUCEDIMAGE = 2
OFILETYPE_PAGE = 3
TIFFTAG_IMAGEWIDTH = 256
TIFFTAG_IMAGELENGTH = 257
TIFFTAG_BITSPERSAMPLE = 258
TIFFTAG_COMPRESSION = 259
COMPRESSION_NONE = 1
COMPRESSION_CCITTRLE = 2
COMPRESSION_CCITTFAX3 = 3
COMPRESSION_CCITT_T4 = 3
COMPRESSION_CCITTFAX4 = 4
COMPRESSION_CCITT_T6 = 4
COMPRESSION_LZW = 5
COMPRESSION_OJPEG = 6
COMPRESSION_JPEG = 7
COMPRESSION_T85 = 9
COMPRESSION_T43 = 10
COMPRESSION_NEXT = 32766
COMPRESSION_CCITTRLEW = 32771
COMPRESSION_PACKBITS = 32773
COMPRESSION_THUNDERSCAN = 32809
COMPRESSION_IT8CTPAD = 32895
COMPRESSION_IT8LW = 32896
COMPRESSION_IT8MP = 32897
COMPRESSION_IT8BL = 32898
COMPRESSION_PIXARFILM = 32908
COMPRESSION_PIXARLOG = 32909
COMPRESSION_DEFLATE = 32946
COMPRESSION_ADOBE_DEFLATE = 8
COMPRESSION_DCS = 32947
COMPRESSION_JBIG = 34661
COMPRESSION_SGILOG = 34676
COMPRESSION_SGILOG24 = 34677
COMPRESSION_JP2000 = 34712
COMPRESSION_LERC = 34887
COMPRESSION_LZMA = 34925
COMPRESSION_ZSTD = 50000
COMPRESSION_WEBP = 50001
TIFFTAG_PHOTOMETRIC = 262
PHOTOMETRIC_MINISWHITE = 0
PHOTOMETRIC_MINISBLACK = 1
PHOTOMETRIC_RGB = 2
PHOTOMETRIC_PALETTE = 3
PHOTOMETRIC_MASK = 4
PHOTOMETRIC_SEPARATED = 5
PHOTOMETRIC_YCBCR = 6
PHOTOMETRIC_CIELAB = 8
PHOTOMETRIC_ICCLAB = 9
PHOTOMETRIC_ITULAB = 10
PHOTOMETRIC_CFA = 32803
PHOTOMETRIC_LOGL = 32844
PHOTOMETRIC_LOGLUV = 32845
TIFFTAG_THRESHHOLDING = 263
THRESHHOLD_BILEVEL = 1
THRESHHOLD_HALFTONE = 2
THRESHHOLD_ERRORDIFFUSE = 3
TIFFTAG_CELLWIDTH = 264
TIFFTAG_CELLLENGTH = 265
TIFFTAG_FILLORDER = 266
FILLORDER_MSB2LSB = 1
FILLORDER_LSB2MSB = 2
TIFFTAG_DOCUMENTNAME = 269
TIFFTAG_IMAGEDESCRIPTION = 270
TIFFTAG_MAKE = 271
TIFFTAG_MODEL = 272
TIFFTAG_STRIPOFFSETS = 273
TIFFTAG_ORIENTATION = 274
ORIENTATION_TOPLEFT = 1
ORIENTATION_TOPRIGHT = 2
ORIENTATION_BOTRIGHT = 3
ORIENTATION_BOTLEFT = 4
ORIENTATION_LEFTTOP = 5
ORIENTATION_RIGHTTOP = 6
ORIENTATION_RIGHTBOT = 7
ORIENTATION_LEFTBOT = 8
TIFFTAG_SAMPLESPERPIXEL = 277
TIFFTAG_ROWSPERSTRIP = 278
TIFFTAG_STRIPBYTECOUNTS = 279
TIFFTAG_MINSAMPLEVALUE = 280
TIFFTAG_MAXSAMPLEVALUE = 281
TIFFTAG_XRESOLUTION = 282
TIFFTAG_YRESOLUTION = 283
TIFFTAG_PLANARCONFIG = 284
PLANARCONFIG_CONTIG = 1
PLANARCONFIG_SEPARATE = 2
TIFFTAG_PAGENAME = 285
TIFFTAG_XPOSITION = 286
TIFFTAG_YPOSITION = 287
TIFFTAG_FREEOFFSETS = 288
TIFFTAG_FREEBYTECOUNTS = 289
TIFFTAG_GRAYRESPONSEUNIT = 290
GRAYRESPONSEUNIT_10S = 1
GRAYRESPONSEUNIT_100S = 2
GRAYRESPONSEUNIT_1000S = 3
GRAYRESPONSEUNIT_10000S = 4
GRAYRESPONSEUNIT_100000S = 5
TIFFTAG_GRAYRESPONSECURVE = 291
TIFFTAG_GROUP3OPTIONS = 292
TIFFTAG_T4OPTIONS = 292
GROUP3OPT_2DENCODING = 1
GROUP3OPT_UNCOMPRESSED = 2
GROUP3OPT_FILLBITS = 4
TIFFTAG_GROUP4OPTIONS = 293
TIFFTAG_T6OPTIONS = 293
GROUP4OPT_UNCOMPRESSED = 2
TIFFTAG_RESOLUTIONUNIT = 296
RESUNIT_NONE = 1
RESUNIT_INCH = 2
RESUNIT_CENTIMETER = 3
TIFFTAG_PAGENUMBER = 297
TIFFTAG_COLORRESPONSEUNIT = 300
COLORRESPONSEUNIT_10S = 1
COLORRESPONSEUNIT_100S = 2
COLORRESPONSEUNIT_1000S = 3
COLORRESPONSEUNIT_10000S = 4
COLORRESPONSEUNIT_100000S = 5
TIFFTAG_TRANSFERFUNCTION = 301
TIFFTAG_SOFTWARE = 305
TIFFTAG_DATETIME = 306
TIFFTAG_ARTIST = 315
TIFFTAG_HOSTCOMPUTER = 316
TIFFTAG_PREDICTOR = 317
PREDICTOR_NONE = 1
PREDICTOR_HORIZONTAL = 2
PREDICTOR_FLOATINGPOINT = 3
TIFFTAG_WHITEPOINT = 318
TIFFTAG_PRIMARYCHROMATICITIES = 319
TIFFTAG_COLORMAP = 320
TIFFTAG_HALFTONEHINTS = 321
TIFFTAG_TILEWIDTH = 322
TIFFTAG_TILELENGTH = 323
TIFFTAG_TILEOFFSETS = 324
TIFFTAG_TILEBYTECOUNTS = 325
TIFFTAG_BADFAXLINES = 326
TIFFTAG_CLEANFAXDATA = 327
CLEANFAXDATA_CLEAN = 0
CLEANFAXDATA_REGENERATED = 1
CLEANFAXDATA_UNCLEAN = 2
TIFFTAG_CONSECUTIVEBADFAXLINES = 328
TIFFTAG_SUBIFD = 330
TIFFTAG_INKSET = 332
INKSET_CMYK = 1
INKSET_MULTIINK = 2
TIFFTAG_INKNAMES = 333
TIFFTAG_NUMBEROFINKS = 334
TIFFTAG_DOTRANGE = 336
TIFFTAG_TARGETPRINTER = 337
TIFFTAG_EXTRASAMPLES = 338
EXTRASAMPLE_UNSPECIFIED = 0
EXTRASAMPLE_ASSOCALPHA = 1
EXTRASAMPLE_UNASSALPHA = 2
TIFFTAG_SAMPLEFORMAT = 339
SAMPLEFORMAT_UINT = 1
SAMPLEFORMAT_INT = 2
SAMPLEFORMAT_IEEEFP = 3
SAMPLEFORMAT_VOID = 4
SAMPLEFORMAT_COMPLEXINT = 5
SAMPLEFORMAT_COMPLEXIEEEFP = 6
TIFFTAG_SMINSAMPLEVALUE = 340
TIFFTAG_SMAXSAMPLEVALUE = 341
TIFFTAG_CLIPPATH = 343
TIFFTAG_XCLIPPATHUNITS = 344
TIFFTAG_YCLIPPATHUNITS = 345
TIFFTAG_INDEXED = 346
TIFFTAG_JPEGTABLES = 347
TIFFTAG_OPIPROXY = 351
TIFFTAG_GLOBALPARAMETERSIFD = 400
TIFFTAG_PROFILETYPE = 401
PROFILETYPE_UNSPECIFIED = 0
PROFILETYPE_G3_FAX = 1
TIFFTAG_FAXPROFILE = 402
FAXPROFILE_S = 1
FAXPROFILE_F = 2
FAXPROFILE_J = 3
FAXPROFILE_C = 4
FAXPROFILE_L = 5
FAXPROFILE_M = 6
TIFFTAG_CODINGMETHODS = 403
CODINGMETHODS_T4_1D = 2
CODINGMETHODS_T4_2D = 4
CODINGMETHODS_T6 = 8
CODINGMETHODS_T85 = 16
CODINGMETHODS_T42 = 32
CODINGMETHODS_T43 = 64
TIFFTAG_VERSIONYEAR = 404
TIFFTAG_MODENUMBER = 405
TIFFTAG_DECODE = 433
TIFFTAG_IMAGEBASECOLOR = 434
TIFFTAG_T82OPTIONS = 435
TIFFTAG_JPEGPROC = 512
JPEGPROC_BASELINE = 1
JPEGPROC_LOSSLESS = 14
TIFFTAG_JPEGIFOFFSET = 513
TIFFTAG_JPEGIFBYTECOUNT = 514
TIFFTAG_JPEGRESTARTINTERVAL = 515
TIFFTAG_JPEGLOSSLESSPREDICTORS = 517
TIFFTAG_JPEGPOINTTRANSFORM = 518
TIFFTAG_JPEGQTABLES = 519
TIFFTAG_JPEGDCTABLES = 520
TIFFTAG_JPEGACTABLES = 521
TIFFTAG_YCBCRCOEFFICIENTS = 529
TIFFTAG_YCBCRSUBSAMPLING = 530
TIFFTAG_YCBCRPOSITIONING = 531
YCBCRPOSITION_CENTERED = 1
YCBCRPOSITION_COSITED = 2
TIFFTAG_REFERENCEBLACKWHITE = 532
TIFFTAG_STRIPROWCOUNTS = 559
TIFFTAG_XMLPACKET = 700
TIFFTAG_OPIIMAGEID = 32781
TIFFTAG_REFPTS = 32953
TIFFTAG_REGIONTACKPOINT = 32954
TIFFTAG_REGIONWARPCORNERS = 32955
TIFFTAG_REGIONAFFINE = 32956
TIFFTAG_MATTEING = 32995
TIFFTAG_DATATYPE = 32996
TIFFTAG_IMAGEDEPTH = 32997
TIFFTAG_TILEDEPTH = 32998
TIFFTAG_PIXAR_IMAGEFULLWIDTH = 33300
TIFFTAG_PIXAR_IMAGEFULLLENGTH = 33301
TIFFTAG_PIXAR_TEXTUREFORMAT = 33302
TIFFTAG_PIXAR_WRAPMODES = 33303
TIFFTAG_PIXAR_FOVCOT = 33304
TIFFTAG_PIXAR_MATRIX_WORLDTOSCREEN = 33305
TIFFTAG_PIXAR_MATRIX_WORLDTOCAMERA = 33306
TIFFTAG_WRITERSERIALNUMBER = 33405
TIFFTAG_CFAREPEATPATTERNDIM = 33421
TIFFTAG_CFAPATTERN = 33422
TIFFTAG_COPYRIGHT = 33432
TIFFTAG_RICHTIFFIPTC = 33723
TIFFTAG_IT8SITE = 34016
TIFFTAG_IT8COLORSEQUENCE = 34017
TIFFTAG_IT8HEADER = 34018
TIFFTAG_IT8RASTERPADDING = 34019
TIFFTAG_IT8BITSPERRUNLENGTH = 34020
TIFFTAG_IT8BITSPEREXTENDEDRUNLENGTH = 34021
TIFFTAG_IT8COLORTABLE = 34022
TIFFTAG_IT8IMAGECOLORINDICATOR = 34023
TIFFTAG_IT8BKGCOLORINDICATOR = 34024
TIFFTAG_IT8IMAGECOLORVALUE = 34025
TIFFTAG_IT8BKGCOLORVALUE = 34026
TIFFTAG_IT8PIXELINTENSITYRANGE = 34027
TIFFTAG_IT8TRANSPARENCYINDICATOR = 34028
TIFFTAG_IT8COLORCHARACTERIZATION = 34029
TIFFTAG_IT8HCUSAGE = 34030
TIFFTAG_IT8TRAPINDICATOR = 34031
TIFFTAG_IT8CMYKEQUIVALENT = 34032
TIFFTAG_FRAMECOUNT = 34232
TIFFTAG_PHOTOSHOP = 34377
TIFFTAG_EXIFIFD = 34665
TIFFTAG_ICCPROFILE = 34675
TIFFTAG_IMAGELAYER = 34732
TIFFTAG_JBIGOPTIONS = 34750
TIFFTAG_GPSIFD = 34853
TIFFTAG_FAXRECVPARAMS = 34908
TIFFTAG_FAXSUBADDRESS = 34909
TIFFTAG_FAXRECVTIME = 34910
TIFFTAG_FAXDCS = 34911
TIFFTAG_STONITS = 37439
TIFFTAG_FEDEX_EDR = 34929
TIFFTAG_INTEROPERABILITYIFD = 40965
TIFFTAG_LERC_PARAMETERS = 50674
TIFFTAG_DNGVERSION = 50706
TIFFTAG_DNGBACKWARDVERSION = 50707
TIFFTAG_UNIQUECAMERAMODEL = 50708
TIFFTAG_LOCALIZEDCAMERAMODEL = 50709
TIFFTAG_CFAPLANECOLOR = 50710
TIFFTAG_CFALAYOUT = 50711
TIFFTAG_LINEARIZATIONTABLE = 50712
TIFFTAG_BLACKLEVELREPEATDIM = 50713
TIFFTAG_BLACKLEVEL = 50714
TIFFTAG_BLACKLEVELDELTAH = 50715
TIFFTAG_BLACKLEVELDELTAV = 50716
TIFFTAG_WHITELEVEL = 50717
TIFFTAG_DEFAULTSCALE = 50718
TIFFTAG_DEFAULTCROPORIGIN = 50719
TIFFTAG_DEFAULTCROPSIZE = 50720
TIFFTAG_COLORMATRIX1 = 50721
TIFFTAG_COLORMATRIX2 = 50722
TIFFTAG_CAMERACALIBRATION1 = 50723
TIFFTAG_CAMERACALIBRATION2 = 50724
TIFFTAG_REDUCTIONMATRIX1 = 50725
TIFFTAG_REDUCTIONMATRIX2 = 50726
TIFFTAG_ANALOGBALANCE = 50727
TIFFTAG_ASSHOTNEUTRAL = 50728
TIFFTAG_ASSHOTWHITEXY = 50729
TIFFTAG_BASELINEEXPOSURE = 50730
TIFFTAG_BASELINENOISE = 50731
TIFFTAG_BASELINESHARPNESS = 50732
TIFFTAG_BAYERGREENSPLIT = 50733
TIFFTAG_LINEARRESPONSELIMIT = 50734
TIFFTAG_CAMERASERIALNUMBER = 50735
TIFFTAG_LENSINFO = 50736
TIFFTAG_CHROMABLURRADIUS = 50737
TIFFTAG_ANTIALIASSTRENGTH = 50738
TIFFTAG_SHADOWSCALE = 50739
TIFFTAG_DNGPRIVATEDATA = 50740
TIFFTAG_MAKERNOTESAFETY = 50741
TIFFTAG_CALIBRATIONILLUMINANT1 = 50778
TIFFTAG_CALIBRATIONILLUMINANT2 = 50779
TIFFTAG_BESTQUALITYSCALE = 50780
TIFFTAG_RAWDATAUNIQUEID = 50781
TIFFTAG_ORIGINALRAWFILENAME = 50827
TIFFTAG_ORIGINALRAWFILEDATA = 50828
TIFFTAG_ACTIVEAREA = 50829
TIFFTAG_MASKEDAREAS = 50830
TIFFTAG_ASSHOTICCPROFILE = 50831
TIFFTAG_ASSHOTPREPROFILEMATRIX = 50832
TIFFTAG_CURRENTICCPROFILE = 50833
TIFFTAG_CURRENTPREPROFILEMATRIX = 50834
TIFFTAG_DCSHUESHIFTVALUES = 65535
TIFFTAG_FAXMODE = 65536
FAXMODE_CLASSIC = 0
FAXMODE_NORTC = 1
FAXMODE_NOEOL = 2
FAXMODE_BYTEALIGN = 4
FAXMODE_WORDALIGN = 8
FAXMODE_CLASSF = 1
TIFFTAG_JPEGQUALITY = 65537
TIFFTAG_JPEGCOLORMODE = 65538
JPEGCOLORMODE_RAW = 0
JPEGCOLORMODE_RGB = 1
TIFFTAG_JPEGTABLESMODE = 65539
JPEGTABLESMODE_QUANT = 1
JPEGTABLESMODE_HUFF = 2
TIFFTAG_FAXFILLFUNC = 65540
TIFFTAG_PIXARLOGDATAFMT = 65549
PIXARLOGDATAFMT_8BIT = 0
PIXARLOGDATAFMT_8BITABGR = 1
PIXARLOGDATAFMT_11BITLOG = 2
PIXARLOGDATAFMT_12BITPICIO = 3
PIXARLOGDATAFMT_16BIT = 4
PIXARLOGDATAFMT_FLOAT = 5
TIFFTAG_DCSIMAGERTYPE = 65550
DCSIMAGERMODEL_M3 = 0
DCSIMAGERMODEL_M5 = 1
DCSIMAGERMODEL_M6 = 2
DCSIMAGERFILTER_IR = 0
DCSIMAGERFILTER_MONO = 1
DCSIMAGERFILTER_CFA = 2
DCSIMAGERFILTER_OTHER = 3
TIFFTAG_DCSINTERPMODE = 65551
DCSINTERPMODE_NORMAL = 0
DCSINTERPMODE_PREVIEW = 1
TIFFTAG_DCSBALANCEARRAY = 65552
TIFFTAG_DCSCORRECTMATRIX = 65553
TIFFTAG_DCSGAMMA = 65554
TIFFTAG_DCSTOESHOULDERPTS = 65555
TIFFTAG_DCSCALIBRATIONFD = 65556
TIFFTAG_ZIPQUALITY = 65557
TIFFTAG_PIXARLOGQUALITY = 65558
TIFFTAG_DCSCLIPRECTANGLE = 65559
TIFFTAG_SGILOGDATAFMT = 65560
SGILOGDATAFMT_FLOAT = 0
SGILOGDATAFMT_16BIT = 1
SGILOGDATAFMT_RAW = 2
SGILOGDATAFMT_8BIT = 3
TIFFTAG_SGILOGENCODE = 65561
SGILOGENCODE_NODITHER = 0
SGILOGENCODE_RANDITHER = 1
TIFFTAG_LZMAPRESET = 65562
TIFFTAG_PERSAMPLE = 65563
PERSAMPLE_MERGED = 0
PERSAMPLE_MULTI = 1
TIFFTAG_ZSTD_LEVEL = 65564
TIFFTAG_LERC_VERSION = 65565
LERC_VERSION_2_4 = 4
TIFFTAG_LERC_ADD_COMPRESSION = 65566
LERC_ADD_COMPRESSION_NONE = 0
LERC_ADD_COMPRESSION_DEFLATE = 1
LERC_ADD_COMPRESSION_ZSTD = 2
TIFFTAG_LERC_MAXZERROR = 65567
TIFFTAG_WEBP_LEVEL = 65568
TIFFTAG_WEBP_LOSSLESS = 65569
EXIFTAG_EXPOSURETIME = 33434
EXIFTAG_FNUMBER = 33437
EXIFTAG_EXPOSUREPROGRAM = 34850
EXIFTAG_SPECTRALSENSITIVITY = 34852
EXIFTAG_ISOSPEEDRATINGS = 34855
EXIFTAG_OECF = 34856
EXIFTAG_EXIFVERSION = 36864
EXIFTAG_DATETIMEORIGINAL = 36867
EXIFTAG_DATETIMEDIGITIZED = 36868
EXIFTAG_COMPONENTSCONFIGURATION = 37121
EXIFTAG_COMPRESSEDBITSPERPIXEL = 37122
EXIFTAG_SHUTTERSPEEDVALUE = 37377
EXIFTAG_APERTUREVALUE = 37378
EXIFTAG_BRIGHTNESSVALUE = 37379
EXIFTAG_EXPOSUREBIASVALUE = 37380
EXIFTAG_MAXAPERTUREVALUE = 37381
EXIFTAG_SUBJECTDISTANCE = 37382
EXIFTAG_METERINGMODE = 37383
EXIFTAG_LIGHTSOURCE = 37384
EXIFTAG_FLASH = 37385
EXIFTAG_FOCALLENGTH = 37386
EXIFTAG_SUBJECTAREA = 37396
EXIFTAG_MAKERNOTE = 37500
EXIFTAG_USERCOMMENT = 37510
EXIFTAG_SUBSECTIME = 37520
EXIFTAG_SUBSECTIMEORIGINAL = 37521
EXIFTAG_SUBSECTIMEDIGITIZED = 37522
EXIFTAG_FLASHPIXVERSION = 40960
EXIFTAG_COLORSPACE = 40961
EXIFTAG_PIXELXDIMENSION = 40962
EXIFTAG_PIXELYDIMENSION = 40963
EXIFTAG_RELATEDSOUNDFILE = 40964
EXIFTAG_FLASHENERGY = 41483
EXIFTAG_SPATIALFREQUENCYRESPONSE = 41484
EXIFTAG_FOCALPLANEXRESOLUTION = 41486
EXIFTAG_FOCALPLANEYRESOLUTION = 41487
EXIFTAG_FOCALPLANERESOLUTIONUNIT = 41488
EXIFTAG_SUBJECTLOCATION = 41492
EXIFTAG_EXPOSUREINDEX = 41493
EXIFTAG_SENSINGMETHOD = 41495
EXIFTAG_FILESOURCE = 41728
EXIFTAG_SCENETYPE = 41729
EXIFTAG_CFAPATTERN = 41730
EXIFTAG_CUSTOMRENDERED = 41985
EXIFTAG_EXPOSUREMODE = 41986
EXIFTAG_WHITEBALANCE = 41987
EXIFTAG_DIGITALZOOMRATIO = 41988
EXIFTAG_FOCALLENGTHIN35MMFILM = 41989
EXIFTAG_SCENECAPTURETYPE = 41990
EXIFTAG_GAINCONTROL = 41991
EXIFTAG_CONTRAST = 41992
EXIFTAG_SATURATION = 41993
EXIFTAG_SHARPNESS = 41994
EXIFTAG_DEVICESETTINGDESCRIPTION = 41995
EXIFTAG_SUBJECTDISTANCERANGE = 41996
EXIFTAG_GAINCONTROL = 41991
EXIFTAG_GAINCONTROL = 41991
EXIFTAG_IMAGEUNIQUEID = 42016
